from typing import Dict
from .location_local_constants import LocationLocalConstants
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.generic_crud_ml import GenericCRUDML  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class County(GenericCRUD, GenericCRUDML):
    def __init__(self):
        logger.start()

        super().__init__(default_schema_name="location")

        # TODO: use GenericCRUD and GenericCRUDML and delete the following 2 lines
        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    # TODO: use GenericCRUD's and GenericCRUDML's insert methods
    def insert(
            self, coordinate: Dict[str, float],
            county: str, lang_code: LangCode, title_approved: bool = False, state_id: int = None) -> int:
        logger.start(object={'coordinate': coordinate, 'county': county,
                     'lang_code': lang_code, 'title_approved': title_approved, 'state_id': state_id})
        insert_county_sql = "INSERT INTO county_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            insert_county_sql, (coordinate["latitude"], coordinate["longitude"]))
        county_id = self.cursor.lastrowid()
        insert_county_ml_sql = "INSERT INTO county_ml_table (county_id, lang_code," \
            " title, title_approved) VALUES (%s, %s, %s, %s)"
        self.cursor.execute(insert_county_ml_sql, (county_id,
                            lang_code, county, title_approved))
        self.connector.commit()
        logger.end(object={'county_id': county_id})
        return county_id

    # TODO: uncomment the commented lines after we add state_id to county_ml_table and county_ml_view
    def get_county_ids_by_county_name_state_id(self, county_name: str, state_id: int = None) -> list[int]:
        logger.start(object={'county_name': county_name, 'state_id': state_id})

        county_ids_dicts_list = self.select_multi_dict_by_where_with_none_option(
            view_table_name=LocationLocalConstants.COUNTY_ML_VIEW_NAME,
            # select_clause_value="county_id, state_id",
            select_clause_value="county_id",
            condition_column_name="title",
            condition_column_value=county_name
        )
        # filtered_county_ids by state_id
        '''
        filtered_county_ids = []
        for county_id in county_ids:
            if county_id["state_id"] == state_id:
                filtered_county_ids.append(county_id["county_id"])

        logger.end(object={'county_ids': county_ids})
        return filtered_county_ids
        '''
        # change dicts to ints
        county_ids = []
        for county_id_dict in county_ids_dicts_list:
            county_ids.append(county_id_dict["county_id"])
        return county_ids



from typing import Dict
from .location_local_constants import LocationLocalConstants
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.generic_crud_ml import GenericCRUDML  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class Neighborhood(GenericCRUD, GenericCRUDML):
    def __init__(self):
        logger.start()

        super().__init__(default_schema_name="location")

        # TODO: use GenericCRUD and GenericCRUDML and delete the following 2 lines
        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    # TODO: use GenericCRUD's and GenericCRUDML's insert methods
    def insert(
            self, coordinate: Dict[str, float],
            neighborhood: str, lang_code: LangCode, title_approved: bool = False, city_id: int = None) -> int:
        logger.start(object={'coordinate': coordinate, 'neighborhood': neighborhood,
                     'lang_code': lang_code, 'title_approved': title_approved, 'city_id': city_id})
        insert_neighborhood_sql = "INSERT INTO neighborhood_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            insert_neighborhood_sql, (coordinate["latitude"], coordinate["longitude"]))
        neighborhood_id = self.cursor.lastrowid()
        insert_neighborhood_ml_sql = "INSERT INTO neighborhood_ml_table (neighborhood_id, lang_code," \
            " title, title_approved) VALUES (%s, %s, %s, %s)"
        self.cursor.execute(insert_neighborhood_ml_sql, (neighborhood_id,
                            lang_code, neighborhood, title_approved))
        self.connector.commit()
        logger.end(object={'neighborhood_id': neighborhood_id})
        return neighborhood_id

    def get_neighborhood_ids_by_neighborhood_name(self, neighborhood_name: str, city_id: int = None) -> list[int]:
        logger.start(object={'neighborhood_name': neighborhood_name, 'city_id': city_id})

        neighborhood_ids_dicts_list = self.select_multi_dict_by_where_with_none_option(
            view_table_name="neighborhood_ml_view",
            select_clause_value="neighborhood_id",
            condition_column_name="title",
            condition_column_value=neighborhood_name
        )

        # change dicts to ints
        neighborhood_ids = []
        for neighborhood_id_dict in neighborhood_ids_dicts_list:
            neighborhood_ids.append(neighborhood_id_dict["neighborhood_id"])

        logger.end(object={'neighborhood_ids': neighborhood_ids})
        return neighborhood_ids



from typing import Dict
from .location_local_constants import LocationLocalConstants
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.generic_crud_ml import GenericCRUDML  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class State(GenericCRUD, GenericCRUDML):
    def __init__(self):
        logger.start()

        super().__init__(default_schema_name="location")

        # TODO: use GenericCRUD and GenericCRUDML and delete the following 2 lines
        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    # TODO: use GenericCRUD's and GenericCRUDML's insert methods
    def insert(
            self, coordinate: Dict[str, float],
            state: str, lang_code: LangCode, state_name_approved: bool = False, country_id: int = None) -> int:
        logger.start(object={'coordinate': coordinate, 'state': state,
                     'lang_code': lang_code, 'state_name_approved': state_name_approved, 'country_id': country_id})
        insert_state_sql = "INSERT INTO state_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            insert_state_sql, (coordinate["latitude"], coordinate["longitude"]))
        state_id = self.cursor.lastrowid()
        insert_state_ml_sql = "INSERT INTO state_ml_table (state_id, lang_code, state_name," \
            " state_name_approved) VALUES (%s, %s, %s, %s)"
        self.cursor.execute(
            insert_state_ml_sql, (state_id, lang_code, state, state_name_approved))
        self.connector.commit()
        logger.end(object={'state_id': state_id})
        return state_id

    def get_state_ids_by_state_name(self, state_name: str, country_id: int = None) -> list[int]:
        logger.start(object={'state_name': state_name, 'country_id': country_id})

        state_ids_dicts_list = self.select_multi_dict_by_where_with_none_option(
            view_table_name="state_ml_view",
            select_clause_value="state_id",
            condition_column_name="state_name",
            condition_column_value=state_name
        )

        # change dicts to ints
        state_ids = []
        for state_id_dict in state_ids_dicts_list:
            state_ids.append(state_id_dict["state_id"])

        logger.end(object={'state_ids': state_ids})
        return state_ids



from shapely.geometry import Point

# TODO Shall we call it OutPoint like OurUrl?
class PointEncoder():

    @staticmethod
    def point_to_coordinates_tuple(obj: Point) -> tuple:
        return (obj.x, obj.y)
    
    @staticmethod
    def point_to_coordinates_str(obj: Point) -> str:
            return f'({obj.x}, {obj.y})'
    
    @staticmethod
    def str_coordinates_to_point(data) -> Point:
        try:
            coordinates = eval(data)
            if isinstance(coordinates, tuple) and len(coordinates) == 2:
                return Point(coordinates[0], coordinates[1])
            else:
                raise ValueError("Invalid coordinates format in the input string.")
        except Exception as e:
            raise ValueError(f"Failed to parse the input string: {e}")